#include <stdio.h>
#include <string.h>

/*
===================================================
        Russian<->EngloRussian Translator
           (c) Alexander Agranov, 1998
===================================================
*/




/* 
 The following tables define russian alphabet in different codepages:
 
 win    - Windows-1251 codepage, used by Win 3.1x/95/98/NT;
 koi    - KOI8-R codepage, used by UNIX and most WWW pages;
 pseudo - some 7-bit pseudo-coding, intended for testing and internal purposes.
*/

char win[64] = {    192,193,194,195,196,197,198,199,200,201,202,
                    203,204,205,206,207,208,209,210,211,212,213,214,
                    215,216,217,218,219,220, 221,222,223  ,
                    
                    224,225,226,227,228,229,230,231,232,233,234,
                    235,236,237,238,239,240,241,242,243,244,245,246,
                    247,248,249,250,251,252 ,253,254,255
               };
               
char koi[64] = {    225,226,247,231,228,229,246,250,233,234,235,
                    236,237,238,239,240,242,243,244,245,230,232,227,
                    254,251,253,255,249,248, 252,224,241  ,
                    
                    193,194,215,199,196,197,214,218,201,202,203,
                    204,205,206,207,208,210,211,212,213,198,200,195,
                    222,219,221,223,217,216, 220,192,209
               }; 

char pseudo[64] = { 'A','B','V','G','D','E','*','Z','I','J','K',
                    'L','M','N','O','P','R','S','T','U','F','H','Q',
                    '&','W','W','~','Y','^', '}','{','|' ,
                    
                    'a','b','v','g','d','e','*','z','i','j','k',
                    'l','m','n','o','p','r','s','t','u','f','h','q',
                    '&','w','w','`','y','\'',']','[','\\'
               };

/*
/////////////////////////////////////////////////////////////////////
//                                                                 //
//         NAME:  rus_eng_xla                                      //
//                                                                 //
//  DESCRIPTION:  Performs recoding from from some russian         //
//                codepage into "EngloRussian" (so-called          //
//                "Volapyuk") coding - i.e. phonetically           //
//                equivalent words in english letters.             //
//                                                                 //
//   PARAMETERS:  char* alf  - source codepage                     //
//                FILE * in  - source file                         //
//                FILE * out - target file                         //
//                                                                 //
// RETURN CODES:  none                                             //
//                                                                 //
//       AUTHOR:  Alexander Agranov                                //
//                                                                 //
/////////////////////////////////////////////////////////////////////
*/
void rus_eng_xla(char* alf, FILE * in, FILE * out)
{
   char ch, flag=0;
   int i;
   
   char * eng[64] = { "A","B","V","G","D","E","Zh","Z","I","J","K",
                      "L","M","N","O","P","R","S","T","U","F","H","Tz",
                      "Ch","Sh","W","`","Y","\'","E","Yu","Ya" ,
                    
                      "a","b","v","g","d","e","zh","z","i","j","k",
                      "l","m","n","o","p","r","s","t","u","f","h","tz",
                      "ch","sh","w","`","y","\'","e","yu","ya"
                    };

   while(!(feof(in)))
   {
      fscanf(in,"%c",&ch);
      
      flag = 0;
      for (i=0;i<64;i++)
      {
         if (alf[i]==ch)  
         { 
            switch (strlen(eng[i]))
            {
               case 1: fprintf(out,"%c",eng[i][0]); break;
               case 2: fprintf(out,"%c%c",eng[i][0],eng[i][1]); break;
               case 3: fprintf(out,"%c%c%c",eng[i][0],eng[i][1],eng[i][2]); break;
            }            
            flag = 1;
         }
      }
      if (flag==0)  fprintf(out,"%c",ch);
   }
}


/*
/////////////////////////////////////////////////////////////////////
//                                                                 //
//         NAME:  eng_rus_xla                                      //
//                                                                 //
//  DESCRIPTION:  Performs recoding from "EngloRussian"            //
//                ("Volapyuk") coding into some russian codepage.  //
//                                                                 //
//   PARAMETERS:  char* alf  - target codepage                     //
//                FILE * in  - source file                         //
//                FILE * out - target file                         //
//                                                                 //
// RETURN CODES:  none                                             //
//                                                                 //
//       AUTHOR:  Alexander Agranov                                //
//                                                                 //
/////////////////////////////////////////////////////////////////////
*/
#define num_cod 21

void eng_rus_xla(char* alf, FILE * in, FILE * out)
{
   char ch, ch1, ch2;
   char trans=0;  /* counter of consequent english letters, that 
                     correspond to one russian letter - starts from 0*/
   char rus=0;    /* alphabet trigger - 0=ENG / 1=RUS */
   char already_printed = 0;
           /* indicates that some character was already printed in main loop */
   int i;
  
   char* cod1[num_cod] = { "ZH", "KH", "TZ", "CH", "SH", "YU", "YA", 
                           "Zh", "Kh", "Tz", "Ch", "Sh", "Yu", "Ya",
                           "zh", "kh", "tz", "ch", "sh", "yu", "ya"
                         };
                         
   char xla1[num_cod] = { 6, 21, 22, 23, 24, 30, 31,
                          6, 21, 22, 23, 24, 30, 31,
                         38, 53, 54, 55, 56, 62, 63
                        };
   
   fscanf(in,"%c",&ch);
   if (feof(in)) return;  /* the input file is empty */
   
   do
   {
      /* 
         THIRD LETTER CHECK - it is not used in existing
         translation tables, but is fully supported by software. 
      */
      if ( (trans==2) && (rus==1) )
      {
         for (i=0;i<num_cod;i++)
         {
            if ( (ch1==cod1[i][0]) && (ch2==cod1[i][1]) )
            {
               if (ch==cod1[i][2])
               {
                  fprintf(out,"%c",alf[xla1[i]]);   /* 3-letter xla */
                  already_printed = 1;
               }
               else   
                  fprintf(out,"%c",alf[xla1[i]]);   /* 2-letter xla */
                  /* 
                     we've printed only two previousely stored letters 
                     (ch1&ch2), while the currently read char still remains 
                     unprocessed - thus we do not set 'already_printed' bool. 
                  */
               trans = 0;
            } 
         }
         /* this shouldn't happen, but anyway */
         if (trans==2)  { fprintf(out,"%c%c",ch1,ch2); trans=0; }
      }

      /* 
         RUSSIAN/ENGLISH SWITCH CONTROL SYMBOL (/e - English; /r - Russian)
         Executing it here, to properly print 2/3-letter translations. 
      */
      if ( (ch=='/') && !already_printed )
      {
         fscanf(in,"%c",&ch);
         if (ch=='r')  
         { 
            rus=1; trans=0; already_printed = 1;
         }
         else
         {
            if (ch=='e')  
            { 
               if (trans==1) fprintf(out,"%c",ch1);
               rus=0; trans=0; already_printed = 1;
            }
            else  fprintf(out,"%c",'/');
         }
      }

      /* 
         SECOND LETTER CHECK - used for such russian letters as CH, SH etc. 
         that phonetically correspond to sequence of 2 english letters.
      */
      if ( (trans==1) && (rus==1) && !already_printed )
      {
         for (i=0;i<num_cod;i++)
         {
            if ( (ch==cod1[i][1]) && (ch1==cod1[i][0])) { trans=2; ch2 = ch; }
         }
         
         if (trans==1) /* it's just one letter */ 
         { 
            for (i=63;i>=0;i--)  /* translating ch1 */
            {
               if ( (ch1==pseudo[i]) && (trans==1) )  { fprintf(out,"%c",alf[i]); trans=0; }
            }
            
            if (trans!=0)  fprintf(out,"%c",ch1); 
            trans=0; 
         }  
      }
      
      /* 
         FIRST LETTER CHECK - let us go through 'cod1' array and set 
         trans=1 if we find first letter of possible sequence.
      */
      if ( (trans==0) && (rus==1) && !already_printed )
      {
         for (i=0;i<num_cod;i++)
         {
            if (ch==cod1[i][0])  { trans=1; ch1 = ch; }
         }
         if (trans==0)  /* simple one letter */
         {
            for (i=63;i>=0;i--)  /* translating ch */
            {
               if ( (ch==pseudo[i]) && (trans==0) )  
               { 
                  if (!already_printed) fprintf(out,"%c",alf[i]); 
                  already_printed = 1; 
               }
            }
            
            if (!already_printed)  fprintf(out,"%c",ch); 
         }
      }
      
      /*
         ENGLISH MODE - no translation at all.
      */
      if (rus==0)  
      {
         if (!already_printed) fprintf(out,"%c",ch);
      }
      
      fscanf(in,"%c",&ch);
      already_printed = 0;
   }
   while(!(feof(in)));

}


/*
/////////////////////////////////////////////////////////////////////
//                                                                 //
//         NAME:  rus_rus_xla                                      //
//                                                                 //
//  DESCRIPTION:  Performs recoding between two russian codepages. //
//                                                                 //
//   PARAMETERS:  char* alf_in   - source codepage                 //
//                char* alf_out  - target codepage                 //
//                FILE * in  - source file                         //
//                FILE * out - target file                         //
//                                                                 //
// RETURN CODES:  none                                             //
//                                                                 //
//       AUTHOR:  Alexander Agranov                                //
//                                                                 //
/////////////////////////////////////////////////////////////////////
*/
void rus_rus_xla(char* alf_in, char* alf_out, FILE * in, FILE * out)
{
   char ch, flag=0;
   int i;
   
   while(!(feof(in)))
   {
      fscanf(in,"%c",&ch);
      
      flag = 0;
      for (i=0;i<64;i++)
      {
         if (alf_in[i]==ch)  
         { 
            fprintf(out,"%c",alf_out[i]);
            flag = 1;
         }
      }
      if (flag==0)  fprintf(out,"%c",ch);
   }
}


/*
/////////////////////////////////////////////////////////////////////
//                                                                 //
//         NAME:  main                                             //
//                                                                 //
//  DESCRIPTION:  Gets command line parameters and executes        //
//                corresponding recoding function.                 //
//                                                                 //
// RETURN CODES:  none                                             //
//                                                                 //
//       AUTHOR:  Alexander Agranov                                //
//                                                                 //
/////////////////////////////////////////////////////////////////////
*/
void main(int argc, char * argv[])
{
   FILE * in;
   FILE * out;
   char flag=0;
   
   if (argc != 5)
   {
      printf("\nRussian<->EngloRussian Translator,  (c) by AGA 1998\n");
      printf("===================================================\n\n");
      printf("AGAXLA <source_file> <source_coding> <dest_file> <dest_coding>\n\n"); 
      printf("Supported codings:\n");
      printf("   ENG - Englo-Russian coding (english letters)\n");
      printf("   WIN - Windows CP-1251 coding\n");
      printf("   KOI - KOI8 (UNIX) coding\n");
      printf("   CYR - Pseudo 7-bit coding (letters+symbols)\n\n");
      return;
   }
      
   if ((in = fopen(argv[1],"rt")) == NULL)
   {
      printf("Can't find source file - %s\n", argv[1]);
      return;
   }
   
   if ((out = fopen(argv[3],"w+t")) == NULL)
   {
      printf("Can't open target file - %s\n", argv[3]);
      return;
   }

   if ( (strcasecmp(argv[2],"eng")==0) && (strcasecmp(argv[4],"koi")==0) )
   {
      eng_rus_xla(koi,in,out);
      flag=1;
   }
   
   if ( (strcasecmp(argv[2],"koi")==0) && (strcasecmp(argv[4],"eng")==0) )
   {
      rus_eng_xla(koi,in,out);
      flag=1;
   }
   
   if ( (strcasecmp(argv[2],"eng")==0) && (strcasecmp(argv[4],"win")==0) )
   {
      eng_rus_xla(win,in,out);
      flag=1;
   }
   
   if ( (strcasecmp(argv[2],"win")==0) && (strcasecmp(argv[4],"eng")==0) )
   {
      rus_eng_xla(win,in,out);
      flag=1;
   }
   
   if ( (strcasecmp(argv[2],"eng")==0) && (strcasecmp(argv[4],"cyr")==0) )
   {
      eng_rus_xla(pseudo,in,out);
      flag=1;
   }
   
   if ( (strcasecmp(argv[2],"cyr")==0) && (strcasecmp(argv[4],"eng")==0) )
   {
      rus_eng_xla(pseudo,in,out);
      flag=1;
   }
   
   if ( (strcasecmp(argv[2],"win")==0) && (strcasecmp(argv[4],"koi")==0) )
   {
      rus_rus_xla(win,koi,in,out);
      flag=1;
   }
   
   if ( (strcasecmp(argv[2],"koi")==0) && (strcasecmp(argv[4],"win")==0) )
   {
      rus_rus_xla(koi,win,in,out);
      flag=1;
   }

   if (flag==0)   printf("Required translation is not supported in this version.\n");

   fclose(out);
   fclose(in);
}
